@@ -1,7 +1,6 @@
-local S = function(s) return s end
-if minetest.global_exists("intllib") then
-	S = intllib.Getter()
-end
+-- support for i18n
+local S = armor_i18n.gettext
+
 local use_moreores = minetest.get_modpath("moreores")
 local function play_sound_effect(player, name)
 	if player then
